package java.lang;

import java.io.OutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.Map;
import java.io.File;
import java.util.List;

public final class ProcessBuilder
{
    private List<String> command;
    private File directory;
    private Map<String, String> environment;
    private boolean redirectErrorStream;
    private ProcessBuilder.Redirect[] redirects;
    
    public ProcessBuilder(List<String> command) {
        if (command == null) {
            throw new NullPointerException();
        }
        this.command = command;
        return;
    }
    
    public ProcessBuilder(String... array) {
        this.command = new ArrayList(array.length);
        for (int length = array.length, i = 0; i < length; ++i) {
            this.command.add(array[i]);
        }
    }
    
    public ProcessBuilder command(List<String> command) {
        if (command == null) {
            throw new NullPointerException();
        }
        this.command = command;
        return this;
    }
    
    public ProcessBuilder command(String... array) {
        this.command = new ArrayList(array.length);
        for (int length = array.length, i = 0; i < length; ++i) {
            this.command.add(array[i]);
        }
        return this;
    }
    
    public List<String> command() {
        return this.command;
    }
    
    public Map<String, String> environment() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("getenv.*"));
        }
        if (this.environment == null) {
            this.environment = ProcessEnvironment.environment();
        }
        assert this.environment != null;
        return this.environment;
    }
    
    ProcessBuilder environment(String[] array) {
        assert this.environment == null;
        if (array != null) {
            this.environment = ProcessEnvironment.emptyEnvironment(array.length);
            assert this.environment != null;
            for (String replaceFirst : array) {
                if (replaceFirst.indexOf(0) != -1) {
                    replaceFirst = replaceFirst.replaceFirst("\u0000.*", "");
                }
                int index = replaceFirst.indexOf(61, 1);
                if (index != -1) {
                    this.environment.put(replaceFirst.substring(0, index), replaceFirst.substring(index + 1));
                }
            }
        }
        return this;
    }
    
    public File directory() {
        return this.directory;
    }
    
    public ProcessBuilder directory(File directory) {
        this.directory = directory;
        return this;
    }
    
    private ProcessBuilder.Redirect[] redirects() {
        if (this.redirects == null) {
            this.redirects = new ProcessBuilder.Redirect[] { ProcessBuilder.Redirect.PIPE, ProcessBuilder.Redirect.PIPE, ProcessBuilder.Redirect.PIPE };
        }
        return this.redirects;
    }
    
    public ProcessBuilder redirectInput(ProcessBuilder.Redirect redirect) {
        if (redirect.type() == ProcessBuilder.Redirect.Type.WRITE || redirect.type() == ProcessBuilder.Redirect.Type.APPEND) {
            throw new IllegalArgumentException("Redirect invalid for reading: " + redirect);
        }
        this.redirects()[0] = redirect;
        return this;
    }
    
    public ProcessBuilder redirectOutput(ProcessBuilder.Redirect redirect) {
        if (redirect.type() == ProcessBuilder.Redirect.Type.READ) {
            throw new IllegalArgumentException("Redirect invalid for writing: " + redirect);
        }
        this.redirects()[1] = redirect;
        return this;
    }
    
    public ProcessBuilder redirectError(ProcessBuilder.Redirect redirect) {
        if (redirect.type() == ProcessBuilder.Redirect.Type.READ) {
            throw new IllegalArgumentException("Redirect invalid for writing: " + redirect);
        }
        this.redirects()[2] = redirect;
        return this;
    }
    
    public ProcessBuilder redirectInput(File file) {
        return this.redirectInput(ProcessBuilder.Redirect.from(file));
    }
    
    public ProcessBuilder redirectOutput(File file) {
        return this.redirectOutput(ProcessBuilder.Redirect.to(file));
    }
    
    public ProcessBuilder redirectError(File file) {
        return this.redirectError(ProcessBuilder.Redirect.to(file));
    }
    
    public ProcessBuilder.Redirect redirectInput() {
        return (this.redirects == null) ? ProcessBuilder.Redirect.PIPE : this.redirects[0];
    }
    
    public ProcessBuilder.Redirect redirectOutput() {
        return (this.redirects == null) ? ProcessBuilder.Redirect.PIPE : this.redirects[1];
    }
    
    public ProcessBuilder.Redirect redirectError() {
        return (this.redirects == null) ? ProcessBuilder.Redirect.PIPE : this.redirects[2];
    }
    
    public ProcessBuilder inheritIO() {
        Arrays.fill(this.redirects(), ProcessBuilder.Redirect.INHERIT);
        return this;
    }
    
    public boolean redirectErrorStream() {
        return this.redirectErrorStream;
    }
    
    public ProcessBuilder redirectErrorStream(boolean redirectErrorStream) {
        this.redirectErrorStream = redirectErrorStream;
        return this;
    }
    
    public Process start() throws IOException {
        String[] array = (String[])((String[])this.command.toArray(new String[this.command.size()])).clone();
        int length = array.length;
        int i = 0;
        while (i < length) {
            if (array[i] == null) {
                throw new NullPointerException();
            }
            ++i;
        }
        String s = array[0];
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkExec(s);
        }
        String s2 = (this.directory == null) ? null : this.directory.toString();
        int j = 1;
        while (j < array.length) {
            if (array[j].indexOf(0) >= 0) {
                throw new IOException("invalid null character in command");
            }
            ++j;
        }
        try {
            return ProcessImpl.start(array, this.environment, s2, this.redirects, this.redirectErrorStream);
        }
        catch (IOException | IllegalArgumentException cause) {
            String string = ": " + cause.getMessage();
            if (cause instanceof IOException && securityManager != null) {
                try {
                    securityManager.checkRead(s);
                }
                catch (SecurityException ex) {
                    string = "";
                    cause = ex;
                }
            }
            throw new IOException("Cannot run program \"" + s + "\"" + ((s2 == null) ? "" : (" (in directory \"" + s2 + "\")")) + string, cause);
        }
    }
    
    static class NullInputStream extends InputStream
    {
        static final ProcessBuilder.NullInputStream INSTANCE;
        
        private NullInputStream() {
        }
        
        @Override
        public int read() {
            return -1;
        }
        
        @Override
        public int available() {
            return 0;
        }
        
        static {
            INSTANCE = new ProcessBuilder.NullInputStream();
        }
    }
    
    static class NullOutputStream extends OutputStream
    {
        static final ProcessBuilder.NullOutputStream INSTANCE;
        
        private NullOutputStream() {
        }
        
        @Override
        public void write(int n) throws IOException {
            throw new IOException("Stream closed");
        }
        
        static {
            INSTANCE = new ProcessBuilder.NullOutputStream();
        }
    }
    
    public abstract static class Redirect
    {
        public static final ProcessBuilder.Redirect PIPE;
        public static final ProcessBuilder.Redirect INHERIT;
        
        public abstract ProcessBuilder.Redirect.Type type();
        
        public File file() {
            return null;
        }
        
        boolean append() {
            throw new UnsupportedOperationException();
        }
        
        public static ProcessBuilder.Redirect from(File file) {
            if (file == null) {
                throw new NullPointerException();
            }
            return new ProcessBuilder.Redirect() {
                @Override
                public ProcessBuilder.Redirect.Type type() {
                    return ProcessBuilder.Redirect.Type.READ;
                }
                
                @Override
                public File file() {
                    return file;
                }
                
                @Override
                public String toString() {
                    return "redirect to read from file \"" + file + "\"";
                }
            };
        }
        
        public static ProcessBuilder.Redirect to(File file) {
            if (file == null) {
                throw new NullPointerException();
            }
            return new ProcessBuilder.Redirect() {
                @Override
                public ProcessBuilder.Redirect.Type type() {
                    return ProcessBuilder.Redirect.Type.WRITE;
                }
                
                @Override
                public File file() {
                    return file;
                }
                
                @Override
                public String toString() {
                    return "redirect to write to file \"" + file + "\"";
                }
                
                @Override
                boolean append() {
                    return false;
                }
            };
        }
        
        public static ProcessBuilder.Redirect appendTo(File file) {
            if (file == null) {
                throw new NullPointerException();
            }
            return new ProcessBuilder.Redirect() {
                @Override
                public ProcessBuilder.Redirect.Type type() {
                    return ProcessBuilder.Redirect.Type.APPEND;
                }
                
                @Override
                public File file() {
                    return file;
                }
                
                @Override
                public String toString() {
                    return "redirect to append to file \"" + file + "\"";
                }
                
                @Override
                boolean append() {
                    return true;
                }
            };
        }
        
        @Override
        public boolean equals(Object o) {
            if (o == this) {
                return true;
            }
            if (!(o instanceof ProcessBuilder.Redirect)) {
                return false;
            }
            ProcessBuilder.Redirect redirect = (ProcessBuilder.Redirect)o;
            if (redirect.type() != this.type()) {
                return false;
            }
            assert this.file() != null;
            return this.file().equals(redirect.file());
        }
        
        @Override
        public int hashCode() {
            File file = this.file();
            if (file == null) {
                return super.hashCode();
            }
            return file.hashCode();
        }
        
        private Redirect() {
        }
        
        static {
            PIPE = new ProcessBuilder.Redirect() {
                @Override
                public ProcessBuilder.Redirect.Type type() {
                    return ProcessBuilder.Redirect.Type.PIPE;
                }
                
                @Override
                public String toString() {
                    return this.type().toString();
                }
            };
            INHERIT = new ProcessBuilder.Redirect() {
                @Override
                public ProcessBuilder.Redirect.Type type() {
                    return ProcessBuilder.Redirect.Type.INHERIT;
                }
                
                @Override
                public String toString() {
                    return this.type().toString();
                }
            };
        }
        
        public enum Type
        {
            PIPE, 
            INHERIT, 
            READ, 
            WRITE, 
            APPEND;
        }
    }
}
